#define PREFIX "#"

struct HISTORY_LINE
{
private:
  int PrevTotal;

  static int GetDigits( double Price )
  {
    int Res = 0;

    while ((bool)(Price = ::NormalizeDouble(Price - (int)Price, 8)))
    {
      Price *= 10;

      Res++;
    }

    return(Res);
  }

  void Create( const long chartID = 0 )
  {
    const int VolumeDigits = HISTORY_LINE::GetDigits(::SymbolInfoDouble(OrderSymbol(), SYMBOL_VOLUME_STEP));
    const int PriceDigits = (int)::SymbolInfoInteger(OrderSymbol(), SYMBOL_DIGITS);
    const bool IsBuy = !OrderType();

    const string ArrowOpen = PREFIX + (string)OrderTicket() + (IsBuy ? " buy " : " sell ") +
                             ::DoubleToString(OrderLots(), VolumeDigits) +
                             " " + OrderSymbol() + " at " + ::DoubleToString(OrderOpenPrice(), PriceDigits);

    const string ArrowClose = ArrowOpen + " close at " + ::DoubleToString(OrderClosePrice(), PriceDigits);

    const string TrendLine = PREFIX + (string)OrderTicket() + " " +
                             ::DoubleToString(OrderOpenPrice(), PriceDigits) + " -> " +
                             ::DoubleToString(OrderClosePrice(), PriceDigits);

    if (::ObjectCreate(chartID, ArrowOpen, IsBuy ? OBJ_ARROW_BUY : OBJ_ARROW_SELL, 0, OrderOpenTime(), OrderOpenPrice()))
      ::ObjectSetInteger(chartID, ArrowOpen, OBJPROP_COLOR, IsBuy ? C'3,95,172' : C'225,68,29');

    if (::ObjectCreate(chartID, ArrowClose, IsBuy ? OBJ_ARROW_SELL : OBJ_ARROW_BUY, 0, OrderCloseTime(), OrderClosePrice()))
      ::ObjectSetInteger(chartID, ArrowClose, OBJPROP_COLOR, IsBuy ? C'225,68,29' : C'3,95,172');

    if (::ObjectCreate(chartID, TrendLine, OBJ_TREND, 0, OrderOpenTime(), OrderOpenPrice(), OrderCloseTime(), OrderClosePrice()))
    {
      ::ObjectSetInteger(chartID, TrendLine, OBJPROP_COLOR, IsBuy ? C'3,95,172' : C'225,68,29');
      ::ObjectSetInteger(chartID, TrendLine, OBJPROP_STYLE, STYLE_DOT);
    }
  }

  bool IsMyName( const string &Str ) const
  {
    return(::StringSubstr(Str, 0, ::StringLen(PREFIX)) == PREFIX);
  }

  void DeleteObjects( const ENUM_OBJECT Type, const long chartID = 0, const int SubWindow = -1 ) const
  {
    for (int i = ::ObjectsTotal(chartID, SubWindow, Type) - 1; i >= 0; i--)
    {
      const string Name = ::ObjectName(chartID, i, SubWindow, Type);

      if (this.IsMyName(Name))
        ::ObjectDelete(chartID, Name);
    }

    return;
  }

public:
  HISTORY_LINE( void ) : PrevTotal(0)
  {
  }

  void Clear( const long chartID = 0 )
  {
    this.DeleteObjects(OBJ_TREND, chartID, 0);
    this.DeleteObjects(OBJ_ARROW_BUY, chartID, 0);
    this.DeleteObjects(OBJ_ARROW_SELL, chartID, 0);

    this.PrevTotal = 0;

    return;
  }

  void Check( const long chartID = 0 )
  {
    const int Total = OrdersHistoryTotal();

    while (this.PrevTotal < Total)
    {
      if (OrderSelect(this.PrevTotal, SELECT_BY_POS, MODE_HISTORY) && (OrderType() <= OP_SELL))
        this.Create(chartID);

      this.PrevTotal++;
    }
  }
};

#undef PREFIX